<?php

/**
 * @file
 * Rules hook implementations for Signup Rules module.
 */

/**
 * Implements hook_rules_event_info().
 */
function signup_rules_rules_event_info() {
  $events = array();
  $events['signup_rules_event_insert'] = array(
    'label' => t('User signed up for content'),
    'module' => 'Signup',
    'arguments' => _signup_rules_event_arguments(),
    'help' => 'Triggered when a user first signs up to content.',
  );
  $events['signup_rules_event_update'] = array(
    'label' => t('Signup was changed'),
    'module' => 'Signup',
    'arguments' => _signup_rules_event_arguments(),
    'help' => 'Triggered when signup data has been updated.',
  );
  $events['signup_rules_event_cancel'] = array(
    'label' => t('Signup was cancelled'),
    'module' => 'Signup',
    'arguments' => _signup_rules_event_arguments(),
    'help' => 'Triggered when a user\'s signup has been cancelled.',
  );
  $events['signup_rules_event_open'] = array(
    'label' => t('Signups were opened for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content to which signups were opened')),
    ),
    'help' => 'Triggered when the status of signups for content is opened.',
  );
  $events['signup_rules_event_close'] = array(
    'label' => t('Signups were closed for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content to which signups were closed')),
    ),
    'help' => 'Triggered when the status of signups for content is closed.',
  );
  return $events;
}

/**
 * Helper function to provide common event arguments.
 */
function _signup_rules_event_arguments() {
  return array(
    'user' => array('type' => 'user', 'label' => t('User who signed up for content')),
    'node' => array('type' => 'node', 'label' => t('Content to which the user is signed up')),
    'signup' => array('type' => 'signup', 'label' => t('Signup data')),
  );
}

/**
 * Implements hook_rules_condition_info().
 *
 * Some of these conditions could have opposites implemented, but that
 * is not necessary since Rules was nice enough to provide an inverted
 * value option.
 * @TODO: possible condition, user signed up from registration form?
 */
function signup_rules_condition_info() {
  $conditions = array();
  $conditions['signup_rules_condition_signup_total'] = array(
    'label' => t('Content signup total has value'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Signup content')),
    ),
    'help' => 'Evaluates to TRUE if the signup total of the content is less than, greater than, or equal to the given numeric value.',
  );
  $conditions['signup_rules_condition_user_is_signed_up'] = array(
    'label' => t('User is signed up to content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content')),
      'user' => array('type' => 'user', 'label' => t('User')),
    ),
    'help' => 'Eveluates to TRUE if the selected user is signed up to content.',
  );
  $conditions['signup_rules_condition_signups_open'] = array(
    'label' => t('Signups are open for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content')),
    ),
    'help' => 'Evaluates to TRUE if signups are open for content.',
  );
  $conditions['signup_rules_condition_signup_attended'] = array(
    'label' => t('Signup is marked attended'),
    'module' => 'Signup',
    'arguments' => array(
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
    ),
    'help' => 'Evaluates to TRUE if a signup has been marked \'attended\'.',
  );
  return $conditions;
}

/**
 * Form for signup total condition.
 *
 * @param $settings
 *   An array of settings for edited forms.
 * @param &$form
 *   The existing form array. This argument is passed by reference so
 *   we just alter it and that's all.
 */
function signup_rules_condition_signup_total_form($settings, &$form) {
  $form['settings']['signup_rules_condition'] = array(
    '#type' => 'select',
    '#title' => t('Signup total is'),
    '#options' => array(
      'greater' => t('greater than'),
      'equal' => t('equal to'),
      'less' => t('less than'),
    ),
    '#default_value' => $settings['signup_rules_condition'],
    '#required' => TRUE,
  );
  $form['settings']['signup_rules_total'] = array(
    '#type' => 'textfield',
    '#title' => t('value'),
    '#default_value' => $settings['signup_rules_total'],
    '#required' => TRUE,
  );
}

/**
 * Validation callback for signup total condition form.
 */
function signup_rules_condition_signup_total_validate($form, $form_state) {
  if (!is_numeric($form_state['values']['settings']['signup_rules_total'])) {
    form_set_error('signup_rules_total', t('You must enter a numeric value.'));
  }
}

/**
 * Condition: determines whether a signup total for a node has a specific value.
 *
 * @param $node
 *   The node being checked.
 * @param $settings
 *   An array of settings from the condition form. This array has values
 *   from our custom elements to use to determine operators and values
 *   against which we check the number of signups for the node.
 */
function signup_rules_condition_signup_total($node, $settings) {
  $total = db_result(db_query("SELECT COUNT(*) FROM {signup_log} WHERE nid = %d", $node->nid));
  switch ($settings['signup_rules_condition']) {
    case 'greater':
      if ($total > $settings['signup_rules_total']) {
        return TRUE;
      }
      return FALSE;
    case 'equal':
      if ($total == $settings['signup_rules_total']) {
        return TRUE;
      }
      return FALSE;
    case 'less':
      if ($total < $settings['signup_rules_total']) {
        return TRUE;
      }
      return FALSE;
  }
}

/**
 * Condition: checks if user is signed up for a node.
 */
function signup_rules_condition_user_is_signed_up($node, $user) {
  $result = db_result(db_query("SELECT uid FROM {signup_log} WHERE uid = %d AND nid = %d", $user->uid, $node->nid));
  return !empty($result);
}

/**
 * Condition: checks if signups are open for a node.
 *
 * We check the signup status of the node by checking the $node
 * object. This object is populated with signup data in hook_nodeapi(),
 * so we should have no problem getting the information here.
 */
function signup_rules_condition_signups_open($node) {
  return $node->signup_status == 1;
}

/**
 * Condition: checks if a signup has been marked attended.
 */
function signup_rules_condition_signup_attended($signup) {
  return $signup->attended == 1;
}

/**
 * Implements hook_rules_action_info().
 */
function signup_rules_action_info() {
  $actions = array();
  $actions['signup_rules_action_cancel_node_signups'] = array(
    'label' => t('Cancel all signups for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content')),
    ),
    'help' => 'Cancels all signups for a piece of content.',
  );
  $actions['signup_rules_action_cancel_user_signups'] = array(
    'label' => t('Cancel all signups for user'),
    'module' => 'Signup',
    'arguments' => array(
      'user' => array('type' => 'user', 'label' => t('User')),
    ),
    'help' => 'Cancels all signups for a given user.',
  );
  $actions['signup_rules_action_open_signups'] = array(
    'label' => t('Open signups for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content')),
    ),
    'help' => 'Opens signups for a piece of content.',
  );
  $actions['signup_rules_action_close_signups'] = array(
    'label' => t('Close signups for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content')),
    ),
    'help' => 'Closes signups for a piece of content.',
  );
  $actions['signup_rules_action_mark_attended'] = array(
    'label' => t('Mark signup as attended'),
    'module' => 'Signup',
    'arguments' => array(
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
    ),
    'base' => 'signup_mark_attended_action',
    'help' => 'Marks a signup as attended.',
  );
  $actions['signup_rules_action_mark_not_attended'] = array(
    'label' => t('Mark signup as not attended'),
    'module' => 'Signup',
    'arguments' => array(
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
    ),
    'base' => 'signup_mark_not_attended_action',
    'help' => 'Marks a signup as not attended.',
  );
  $actions['signup_rules_action_signup_user'] = array(
    'label' => t('Sign up user to content'),
    'module' => 'Signup',
    'arguments' => array(
      'user' => array('type' => 'user', 'label' => t('User')),
      'node' => array('type' => 'node', 'label' => t('Content')),
    ),
    'help' => 'Signs up a user to content.',
  );
  $actions['signup_rules_action_cancel_signup'] = array(
    'label' => t('Cancel signup'),
    'module' => 'Signup',
    'arguments' => array(
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
    ),
    'base' => 'signup_cancel_action',
    'help' => 'Cancels an individual signup.',
  );
  $actions['signup_rules_action_mail_to_signup_users'] = array(
    'label' => t('Send mail to all users signed up for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content')),
    ),
    'eval input' => array('subject', 'message'),
    'help' => 'Sends a mail to all users signed up for content.',
  );
  return $actions;
}

/**
 * Action: cancels all signups for a node.
 */
function signup_rules_action_cancel_node_signups($node, $settings) {
  $signups = signup_get_signups($node->nid);
  foreach ($signups as $signup) {
    signup_cancel_signup($signup, FALSE);
  }
  return;
}

/**
 * Action: cancel all signups for a user.
 */
function signup_rules_action_cancel_user_signups($user) {
  $signups = signup_rules_get_user_signups($user->uid);
  foreach ($signups as $signup) {
    signup_cancel_signup($signup, FALSE);
  }
}

/**
 * Action: closes signups on a node.
 *
 * The conditional checks to see if signups are even enabled for
 * the node type. If they aren't then there is no attempt to close
 * them as they should be closed by Signup already.
 */
function signup_rules_action_close_signups($node) {
  if (variable_get('signup_node_default_state_'. $node->type, 'disabled') != 'disabled') {
    signup_close_signup($node->nid);
  }
}

/**
 * Action: opens signups on a node.
 *
 * The conditional checks to ensure that signups are enabled for
 * the node before programatically opening signups.
 */
function signup_rules_action_open_signups($node) {
  if (variable_get('signup_node_default_state_'. $node->type, 'disabled') != 'disabled') {
    signup_open_signup($node->nid);
  }
}

/**
 * Action: signs up a user to content.
 */
function signup_rules_action_signup_user($user, $node) {
  // Check if signups are enabled for the node.
  $status = db_result(db_query('SELECT status FROM {signup} WHERE nid = %d', $node->nid));
  if ($status == 1) {
    $data['nid'] = $node->nid;
    $data['uid'] = $user->uid;
    signup_sign_up_user($data);
  }
}

/**
 * Action: Send mail to all users signed up for content.
 */
function signup_rules_action_mail_to_signup_users($node, $settings) {
  $nid = $node->nid;
  $subject = str_replace(array("\r", "\n"), '', $settings['subject']);
  $body = $settings['message'];
  $from = variable_get('site_mail', ini_get('sendmail_from'));
  module_load_include('inc', 'signup', 'includes/broadcast');
  signup_send_broadcast($nid, $subject, $body, $from);
  watchdog('signup_rules', 'Successfully sent email to users signed up for %title.', array('%title' => $node->title), WATCHDOG_NOTICE, l(t('view'), 'node/'. $node->nid));
}

/**
 * Action "Send mail to all users signed up for content" configuration form.
 *
 * @see signup_broadcast_form()
 */
function signup_rules_action_mail_to_signup_users_form($settings = array(), &$form) {
  $settings += array('subject' => '', 'message' => '');
  $form['settings']['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => $settings['subject'],
    '#description' => t("The mail's subject."),
  );
  
  $tokens = array('%node_title', '%node_url', '%user_mail', '%user_name', t('%cancel_signup_url (access to this link is denied to users without the "cancel own signups" permission)'));

  // We don't have the node object so can't check _signup_get_node_scheduler().
  // Load these anyway to cover 99% of cases that use a signup scheduler.
  $tokens = array_merge(array('%node_start_time'), $tokens);

  if (module_exists('token')) {
    $token_text = t('Supported string substitutions: %tokens, and any tokens in the %replacement_tokens list.', array('%tokens' => implode(', ', $tokens), '%replacement_tokens' => t('Replacement tokens')));
  }
  else {
    $token_text = t('Supported string substitutions: %tokens.', array('%tokens' => implode(', ', $tokens)));
  }
  
  $form['settings']['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
    '#description' => t('Body of the email message you wish to send to all users who have signed up for this content. !token_description', array('!token_description' => $token_text)),
  );
}

/**
 * Implements hook_rules_date_type_info().
 *
 * Tokens are made available by Signup module, so we make use of their
 * Token type. Also, our data type class makes use of save and load
 * functions, so this data type is saveable.
 */
function signup_rules_data_type_info() {
  return array(
    'signup' => array(
      'label' => t('Signup data'),
      'class' => 'rules_data_type_signup',
      'module' => 'Signup',
      'token type' => 'signup',
      'identifiable' => TRUE,
    ),
  );
}

/**
 * Defines the signup data type.
 */
class rules_data_type_signup extends rules_data_type {

  function save() {
    $signup = &$this->get();
    signup_save_signup($signup);
    return TRUE;
  }

  function load($sid) {
    return signup_load_signup($sid);
  }

  function get_identifier() {
    $signup = $this->get();
    return $signup->sid;
  }
}
